==> Botania-1.19.2-438-FABRIC-20230214.034242-11-sources.jar: vazkii/botania/common/item/relic/DiceOfFateItem.java <==
/*
 * This class is distributed as part of the Botania Mod.
 * Get the Source Code in github:
 * https://github.com/Vazkii/Botania
 *
 * Botania is Open Source and distributed under the
 * Botania License: http://botaniamod.net/license.php
 */
package vazkii.botania.common.item.relic;

import com.google.common.base.Suppliers;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import vazkii.botania.api.item.Relic;
import vazkii.botania.client.gui.TooltipHandler;
import vazkii.botania.common.handler.BotaniaSounds;
import vazkii.botania.common.helper.PlayerHelper;
import vazkii.botania.common.item.BotaniaItems;
import vazkii.botania.common.lib.ResourceLocationHelper;
import vazkii.botania.xplat.XplatAbstractions;

import java.util.ArrayList;
import java.util.List;
import java.util.function.Supplier;
import net.minecraft.class_124;
import net.minecraft.class_1268;
import net.minecraft.class_1271;
import net.minecraft.class_1657;
import net.minecraft.class_173;
import net.minecraft.class_1799;
import net.minecraft.class_181;
import net.minecraft.class_1836;
import net.minecraft.class_1937;
import net.minecraft.class_2561;
import net.minecraft.class_2960;
import net.minecraft.class_3218;
import net.minecraft.class_3222;
import net.minecraft.class_3419;
import net.minecraft.class_47;
import net.minecraft.class_52;

public class DiceOfFateItem extends RelicItem {
	public DiceOfFateItem(class_1793 props) {
		super(props);
	}

	public static final Supplier<List<class_1799>> RELIC_STACKS = Suppliers.memoize(() -> List.of(
			new class_1799(BotaniaItems.infiniteFruit),
			new class_1799(BotaniaItems.kingKey),
			new class_1799(BotaniaItems.flugelEye),
			new class_1799(BotaniaItems.thorRing),
			new class_1799(BotaniaItems.odinRing),
			new class_1799(BotaniaItems.lokiRing)
	)
	);

	@NotNull
	@Override
	public class_1271<class_1799> method_7836(class_1937 world, class_1657 player, @NotNull class_1268 hand) {
		class_1799 stack = player.method_5998(hand);
		var relic = XplatAbstractions.INSTANCE.findRelic(stack);

		if (relic != null && relic.isRightPlayer(player)) {
			if (world.field_9236) {
				return class_1271.method_22427(stack);
			}

			world.method_43128(null, player.method_23317(), player.method_23318(), player.method_23321(), BotaniaSounds.diceOfFate, class_3419.field_15248, 1F, 0.4F / (world.field_9229.method_43057() * 0.4F + 0.8F));

			List<Integer> possible = new ArrayList<>();
			for (int i = 0; i < 6; i++) {
				if (!hasRelicAlready(player, i)) {
					possible.add(i);
				}
			}

			if (!possible.isEmpty()) {
				int relicIdx = possible.get(world.field_9229.method_43048(possible.size()));
				player.method_43496(class_2561.method_43469("botaniamisc.diceRoll", relicIdx + 1).method_27692(class_124.field_1077));
				var toGive = RELIC_STACKS.get().get(relicIdx).method_7972();
				return class_1271.method_22428(toGive);
			} else {
				int roll = world.field_9229.method_43048(6) + 1;
				class_2960 tableId = ResourceLocationHelper.prefix("dice/roll_" + roll);
				class_52 table = world.method_8503().method_3857().method_367(tableId);
				class_47 context = new class_47.class_48(((class_3218) world))
						.method_312(class_181.field_1226, player)
						.method_312(class_181.field_24424, player.method_19538())
						.method_303(player.method_7292())
						.method_309(class_173.field_16235);

				List<class_1799> generated = table.method_319(context);
				for (class_1799 drop : generated) {
					player.method_31548().method_7398(drop);
				}
				String langKey = generated.isEmpty() ? "botaniamisc.dudDiceRoll" : "botaniamisc.diceRoll";
				player.method_43496(class_2561.method_43469(langKey, roll).method_27692(class_124.field_1077));

				stack.method_7934(1);
				return class_1271.method_22428(stack);
			}
		}

		return class_1271.method_22430(stack);
	}

	@Override
	public void method_7851(class_1799 stack, @Nullable class_1937 world, List<class_2561> tooltip, class_1836 flags) {
		super.method_7851(stack, world, tooltip, flags);
		tooltip.add(class_2561.method_43470(""));
		TooltipHandler.addOnShift(tooltip, () -> {
			String name = stack.method_7922() + ".poem";
			for (int i = 0; i < 4; i++) {
				tooltip.add(class_2561.method_43471(name + i).method_27695(class_124.field_1080, class_124.field_1056));
			}
		});
	}

	public static Relic makeRelic(class_1799 stack) {
		return new RelicImpl(stack, null) {
			@Override
			public boolean shouldDamageWrongPlayer() {
				return false;
			}
		};
	}

	private boolean hasRelicAlready(class_1657 player, int relicId) {
		if (relicId < 0 || relicId > 6 || !(player instanceof class_3222 mpPlayer)) {
			return true;
		}

		var stack = RELIC_STACKS.get().get(relicId);
		var relic = XplatAbstractions.INSTANCE.findRelic(stack);

		if (relic != null && relic.getAdvancement() != null) {
			return PlayerHelper.hasAdvancement(mpPlayer, relic.getAdvancement());
		}

		return false;
	}

}

==> Botania-1.20.1-451-FABRIC-20251019.112919-17-sources.jar: vazkii/botania/common/item/relic/DiceOfFateItem.java <==
/*
 * This class is distributed as part of the Botania Mod.
 * Get the Source Code in github:
 * https://github.com/Vazkii/Botania
 *
 * Botania is Open Source and distributed under the
 * Botania License: http://botaniamod.net/license.php
 */
package vazkii.botania.common.item.relic;

import com.google.common.base.Suppliers;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import vazkii.botania.api.item.Relic;
import vazkii.botania.client.gui.TooltipHandler;
import vazkii.botania.common.handler.BotaniaSounds;
import vazkii.botania.common.helper.PlayerHelper;
import vazkii.botania.common.item.BotaniaItems;
import vazkii.botania.common.lib.ResourceLocationHelper;
import vazkii.botania.xplat.XplatAbstractions;

import java.util.ArrayList;
import java.util.List;
import java.util.function.Supplier;
import net.minecraft.class_124;
import net.minecraft.class_1268;
import net.minecraft.class_1271;
import net.minecraft.class_1657;
import net.minecraft.class_173;
import net.minecraft.class_1799;
import net.minecraft.class_181;
import net.minecraft.class_1836;
import net.minecraft.class_1937;
import net.minecraft.class_2561;
import net.minecraft.class_2960;
import net.minecraft.class_3218;
import net.minecraft.class_3222;
import net.minecraft.class_3419;
import net.minecraft.class_52;
import net.minecraft.class_8567;

public class DiceOfFateItem extends RelicItem {
	public DiceOfFateItem(class_1793 props) {
		super(props);
	}

	public static final Supplier<List<class_1799>> RELIC_STACKS = Suppliers.memoize(() -> List.of(
			new class_1799(BotaniaItems.infiniteFruit),
			new class_1799(BotaniaItems.kingKey),
			new class_1799(BotaniaItems.flugelEye),
			new class_1799(BotaniaItems.thorRing),
			new class_1799(BotaniaItems.odinRing),
			new class_1799(BotaniaItems.lokiRing)
	)
	);

	@NotNull
	@Override
	public class_1271<class_1799> method_7836(class_1937 world, class_1657 player, @NotNull class_1268 hand) {
		class_1799 stack = player.method_5998(hand);
		var relic = XplatAbstractions.INSTANCE.findRelic(stack);

		if (relic != null && relic.isRightPlayer(player)) {
			if (world.field_9236) {
				return class_1271.method_22427(stack);
			}

			world.method_43128(null, player.method_23317(), player.method_23318(), player.method_23321(), BotaniaSounds.diceOfFate, class_3419.field_15248, 1F, 0.4F / (world.field_9229.method_43057() * 0.4F + 0.8F));

			List<Integer> possible = new ArrayList<>();
			for (int i = 0; i < 6; i++) {
				if (!hasRelicAlready(player, i)) {
					possible.add(i);
				}
			}

			if (!possible.isEmpty()) {
				int relicIdx = possible.get(world.field_9229.method_43048(possible.size()));
				player.method_43496(class_2561.method_43469("botaniamisc.diceRoll", relicIdx + 1).method_27692(class_124.field_1077));
				var toGive = RELIC_STACKS.get().get(relicIdx).method_7972();
				return class_1271.method_22428(toGive);
			} else {
				int roll = world.field_9229.method_43048(6) + 1;
				class_2960 tableId = ResourceLocationHelper.prefix("dice/roll_" + roll);
				class_52 table = world.method_8503().method_3857().getLootTable(tableId);
				class_8567 context = new class_8567.class_8568((class_3218) world)
						.method_51874(class_181.field_1226, player)
						.method_51874(class_181.field_24424, player.method_19538())
						.method_51871(player.method_7292())
						.method_51875(class_173.field_16235);

				List<class_1799> generated = table.method_51878(context);
				for (class_1799 drop : generated) {
					player.method_31548().method_7398(drop);
				}
				String langKey = generated.isEmpty() ? "botaniamisc.dudDiceRoll" : "botaniamisc.diceRoll";
				player.method_43496(class_2561.method_43469(langKey, roll).method_27692(class_124.field_1077));

				stack.method_7934(1);
				return class_1271.method_22428(stack);
			}
		}

		return class_1271.method_22430(stack);
	}

	@Override
	public void method_7851(class_1799 stack, @Nullable class_1937 world, List<class_2561> tooltip, class_1836 flags) {
		super.method_7851(stack, world, tooltip, flags);
		tooltip.add(class_2561.method_43470(""));
		TooltipHandler.addOnShift(tooltip, () -> {
			String name = stack.method_7922() + ".poem";
			for (int i = 0; i < 4; i++) {
				tooltip.add(class_2561.method_43471(name + i).method_27695(class_124.field_1080, class_124.field_1056));
			}
		});
	}

	public static Relic makeRelic(class_1799 stack) {
		return new RelicImpl(stack, null) {
			@Override
			public boolean shouldDamageWrongPlayer() {
				return false;
			}
		};
	}

	private boolean hasRelicAlready(class_1657 player, int relicId) {
		if (relicId < 0 || relicId > 6 || !(player instanceof class_3222 mpPlayer)) {
			return true;
		}

		var stack = RELIC_STACKS.get().get(relicId);
		var relic = XplatAbstractions.INSTANCE.findRelic(stack);

		if (relic != null && relic.getAdvancement() != null) {
			return PlayerHelper.hasAdvancement(mpPlayer, relic.getAdvancement());
		}

		return false;
	}

}
